#include "Case.h"

#include "CaseBase.h"

#include <cassert>

using namespace std;
//using namespace TCBR_Representation;

namespace TCBR_Representation
{

Entity* caseEntity = NULL;

Case::Case(string name, CaseBase* cb):
Entity(name, cb)
{
	if (!caseEntity) {
		caseEntity = new Entity("CASE", cb);
	}

	this->addRelation(new Relation (this, caseEntity, "INSTANCE_OF", 1.0));
	caseEntity->addRelation(new Relation(caseEntity, this, "HAS_INSTANCE", 1.0));
}

Case::~Case()
{
}

void Case::addFeature(string name, string value)
{
	//Relation* rel;
	KnowledgeModel* km = getKnowledgeModel();
	Feature* feature = new Feature(name, value, km);

	//Relate the feature entity to the case
	this->addRelation(new Relation(this, feature, "HAS_FINDING", 1.0));
	feature->addRelation(new Relation(feature, this, "FINDING_OF", 1.0));
}

void Case::addFeature(Feature* feature)
{
	Relation* rel = new Relation(this, feature, "HAS_FINDING", 1.0);
	this->addRelation(rel);

	rel = new Relation(feature, this, "FINDING_OF", 1.0);
	feature->addRelation(rel);
}

vector<Feature*> Case::getFeatures()
{
	const vector<Relation*>& rels = this->getLocalRelations("HAS_FINDING");

	vector<Feature*> findings;
	for (vector<Relation*>::const_iterator it = rels.begin(); it != rels.end(); ++it) {
		Feature* feature = dynamic_cast<Feature*>((*it)->getTarget());
		assert(feature);
		findings.push_back(feature);
	}
	return findings;
}

Feature* Case::getFeature(string type)
{
	const vector<Feature*>& features = getFeatures();

	for (unsigned int i = 0; i < features.size(); i++) {
		if (features[i]->getType()->getName() == type) {
			return features[i];
		}
	}
	return NULL;;
}

bool Case::hasFeature(string type)
{
	return (this->getFeature(type) != NULL);
}

/*
void Case::printContent()
{
	cout << "Printing case: " << getName() << endl;

	Entity* thisCase = getKnowledgeModel()->getEntity(getName());
	vector<Relation*> rels = thisCase->getLocalRelations("HAS_FINDING");

	for (unsigned int i = 0; i < rels.size(); i++)
	{
		rels.at(i)->printContent();//getTarget()->printContent();
	}
}
*/

// my try at making a printout of features and feature values of a case
void Case::printContent()
{
	cout << "Printing features of case: " << getName() << endl;

	vector<Feature*> features = getFeatures();

	for (unsigned int i = 0; i < features.size(); i++)
	{
		cout << "Feature " << i+1 << ": " << features[i]->getType()->getName() << endl;
		cout << "Value: " << getFeature(features[i]->getType()->getName())->getValue()->getTextValue() << endl;

	}
}


//This function removes the relation to the feature
//TODO check whether the feature has other cases pointing to it
//IF NOT -> remove it completely => tidy up a bit
void Case::removeFeature(string type)
{
	Entity* thisCase = getKnowledgeModel()->getEntity(getName());
	vector<Relation*> rels = thisCase->getLocalRelations("HAS_FINDING");

	for (vector<Relation*>::iterator it = rels.begin(); it != rels.end(); ++it) {
		Feature* feature = dynamic_cast<Feature*>((*it)->getTarget());
		assert(feature);
		if (feature->getType()->getName() == type) {
			rels.erase(it);
			break;
		}
	}
}
}
