#include "CaseBase.h"

#include "SimilarityMeasure.h"

#include <memory>
#include <cassert>

using namespace std;

namespace TCBR_Representation
{

CaseBase::CaseBase(string name):
KnowledgeModel(name)
{
}

CaseBase::~CaseBase()
{
}

// Retrieves the cases in the case base
set<Case*> CaseBase::getCases()
{
	set<Case*> result;
	Entity* caseEntity = getEntity("CASE");
	vector<Relation*> relations(caseEntity->getLocalRelations("HAS_INSTANCE"));
	for (vector<Relation*>::const_iterator it = relations.begin(); it != relations.end(); ++it) {
		Case* c = dynamic_cast<Case*>((*it)->getTarget());
		assert(c);
		result.insert(c);
	}
	return result;
}

set<Entity*> CaseBase::getFeatureTypes()
{
	set<Entity*> result;
	Entity* featureEntity = getEntity("FEATURE");
	vector<Relation*> relations(featureEntity->getLocalRelations("HAS_INSTANCE"));
	for (vector<Relation*>::const_iterator it = relations.begin(); it != relations.end(); ++it) {
		Feature* f = dynamic_cast<Feature*>((*it)->getTarget());
		assert(f);
		result.insert(f->getType());
	}
	return result;
}

void CaseBase::setWeight(const string& type, double weight) {
	stringstream ss;
	ss << weight;
	Value* valueEntity = new Value(ss.str(), this);
	Entity* typeEntity = this->getEntity(type);

	typeEntity->addRelation(new Relation(typeEntity, valueEntity, "HAS_WEIGHT", 1.0));

	valueEntity->addRelation(new Relation(valueEntity, typeEntity, "WEIGHT_OF", 1.0));
}

double getDoubleValue(Value* value)
{
	double weight;
	stringstream ss;
	ss << value->getTextValue();
	ss >> weight;
	return weight;
}

double CaseBase::getWeight(const string& type) {
	Entity* typeEntity = this->getEntity(type);
	assert(typeEntity);
	const vector<Relation*>& rels = typeEntity->getLocalRelations("HAS_WEIGHT");

	if (rels.size() == 0) {
		// No custom weight assigned
		return 1.0;
	}

	assert(rels.size() == 1);
	Value* target = dynamic_cast<Value*>(rels.front()->getTarget());
	assert(target);
	double weight = getDoubleValue(target);
	return weight;
}

}
