#include "CaseRetriever.h"

#include "CaseBase.h"
#include "SimilarityMeasure.h"

#include <memory>
#include <cassert>
#include <map>

using namespace std;

namespace TCBR_Representation
{

CaseRetriever::CaseRetriever(CaseBase* caseBase): caseBase_(caseBase) {}

SimilarityMeasureCaseRetriever::SimilarityMeasureCaseRetriever(CaseBase* caseBase, SimilarityMeasure* similarityMeasure): CaseRetriever(caseBase), similarityMeasure_(similarityMeasure) {}

struct gtcasepair
{
	bool operator()(const pair<Case*, double>& a, const pair<Case*, double>& b) const {
		return a.second > b.second;
	}
};

// Returns the cases in the case base sorted by similarity to the input case
vector<pair<Case*, double> > SimilarityMeasureCaseRetriever::retrieveSimilarCases(Case* inputCase)
{
	vector<pair<Case*, double> > result;

	set<Case*> cases(caseBase_->getCases());

	for (set<Case*>::const_iterator it = cases.begin(); it != cases.end(); ++it) {
		if (*it != inputCase) {
			double similarity = similarityMeasure_->computeCaseSimilarity(inputCase, *it);
			result.push_back(make_pair(*it, similarity));
		}
	}

	sort(result.begin(), result.end(), gtcasepair());

	return result;
}

}
