#include "Entity.h"

#include "KnowledgeModel.h"
#include "Relation.h"

namespace TCBR_Representation
{
using namespace std;

Entity* entityEntity = NULL;

Entity::Entity (string name, KnowledgeModel* knowledgeModel):
name_(name), knowledgeModel_(knowledgeModel)
{
	knowledgeModel_->addEntity(this);

	if (!entityEntity) {
		if (name == "ENTITY") {
			entityEntity = this;
		} else {
			new Entity ("ENTITY", knowledgeModel_);
		}
	}

	if (entityEntity != this) {
		Entity* e = knowledgeModel_->getEntity("ENTITY");
		Relation* rel = new Relation(this, e, "INSTANCE_OF", 1.0);
		addRelation(rel);

		rel = new Relation (e, this, "HAS_INSTANCE", 1.0);
		e->addRelation(rel);
	}
}

Entity::~Entity()
{
}

void Entity::addRelation(Relation* r)
{
	relations_.insert(make_pair(r->getType()->getName(), r));
}

const multimap<string, Relation*>& Entity::getLocalRelations()
{
	return relations_;
}

vector<Relation*> Entity::getLocalRelations(const string& type)
{
	const multimap<string, Relation*>& rels = this->getLocalRelations();
	vector<Relation*> typeRels;

	multimap<string, Relation*>::const_iterator it = rels.find(type);
	while (it != rels.end() && it->first == type) {
		typeRels.push_back(it->second);
		++it;
	}

	return typeRels;
}

vector<Relation*> Entity::getLocalRelations(const Entity* type)
{
	return getLocalRelations(type->getName());
}

const string& Entity::getName() const
{
	return name_;
}

KnowledgeModel* Entity::getKnowledgeModel()
{
	return knowledgeModel_;
}

bool Entity::equals(Entity* e)
{
	return (e->getName() == getName());
}

void Entity::printContent()
{
	cout << "Printing entity: " << getName() <<endl;
	Entity* e = getKnowledgeModel()->getEntity(getName());
	vector<Relation*> rels = e->getLocalRelations("TYPE_OF");

	for (unsigned int i = 0; i < rels.size(); i++) {
		rels.at(i)->printContent();
	}
}
};
