#include "Feature.h"

#include <cassert>

using namespace std;

namespace TCBR_Representation
{

int Feature::featureID_	= 0;

Entity* featureEntity = NULL;

Feature::Feature(string type, string value, KnowledgeModel* knowledgeModel):
Entity (generateID(featureID_), knowledgeModel)
{
	if (!featureEntity) {
		if (!knowledgeModel->hasEntity("FEATURE")) {
			featureEntity = new Entity("FEATURE", knowledgeModel);
		} else {
			featureEntity = knowledgeModel->getEntity("FEATURE");
		}
	}

	featureEntity->addRelation(new Relation(featureEntity, this, "HAS_INSTANCE", 1.0));
	this->addRelation(new Relation(this, featureEntity, "INSTANCE_OF", 1.0));

	//Make relations to entity type
	Entity* featureType = knowledgeModel->getEntity(type);
	if (!featureType) {
		featureType = new Entity(type, knowledgeModel);
	}

	this->addRelation(new Relation(this, featureType, "HAS_TYPE", 1.0));
	featureType->addRelation(new Relation(featureType, this, "TYPE_OF", 1.0));

	addValue(value);
	featureID_++;
}

Feature::Feature(string name, KnowledgeModel* knowledgeModel):
Entity(name, knowledgeModel)
{
}

Feature::~Feature()
{
}

//Should throw AlreadyHasValueException
void Feature::addValue(string value)
{
	Value* valueObject = new Value(value, getKnowledgeModel());
	this->addRelation(new Relation(this, valueObject, "HAS_VALUE", 1.0));
	valueObject->addRelation(new Relation(valueObject, this, "VALUE_OF", 1.0));
}

//Should throw HasNoValueException
Value* Feature::getValue()
{
	Value* value = NULL;
	Entity* thisFeature = getKnowledgeModel()->getEntity(getName());
	vector<Relation*> rels = thisFeature->getLocalRelations("HAS_VALUE");

	if (rels.size() != 0) {
		value = dynamic_cast<Value*>(rels.front()->getTarget());
		assert(value);
	}
	return value;
}

bool Feature::hasValue()
{
	return (this->getValue() != NULL);
}

Entity* Feature::getType()
{
	const vector<Relation*>& rels = this->getLocalRelations("HAS_TYPE");
	if (rels.size() > 0) {
		return rels.front()->getTarget();
	} else {
		return NULL;
	}
}

Entity* Feature::getOwner()
{
	const vector<Relation*>& rels = this->getLocalRelations("FINDING_OF");
	if (rels.size() > 0) {
		return rels.front()->getTarget();
	} else {
		return NULL;
	}
}

void Feature::printContent()
{
	cout << "Printing feature: " << getName() << endl;
	cout << "Type " << getType()->getName() << endl;

	if (hasValue()) {
		getValue()->printContent();
	}
}
}
