#include "KnowledgeModel.h"

#include "Entity.h"
#include "Relation.h"

namespace TCBR_Representation
{

using namespace std;

KnowledgeModel::KnowledgeModel(string name): name_(name)
{
}

KnowledgeModel::~KnowledgeModel()
{
	for (unsigned int i = 0; i < relations_.size(); i++)
	{
		delete relations_[i];
	}

	for (map<string, Entity*>::const_iterator it = entities_.begin() ; it != entities_.end(); ++it)
	{
		delete it->second;
	}
}

void KnowledgeModel::addEntity(Entity* e)
{
	entities_[e->getName()] = e;
}

void KnowledgeModel::addRelation(Relation* r)
{
	relations_.push_back(r);
}

vector<Relation*>* KnowledgeModel::getRelations(const std::string type)
{
	vector<Relation*>* rels = new vector<Relation*>();

	unsigned int i;
	for (i = 0; i < relations_.size(); i++)
	{
		if (relations_[i]->getType()!= NULL)
		{
			if (relations_[i]->getType()->getName().compare(type)==0)
			{
				rels->push_back(relations_[i]);
			}
		}
	}
	return rels;
}

vector<Relation*>* KnowledgeModel::getRelations(Entity* type)
{
	return getRelations(type->getName());
}

Entity* KnowledgeModel::getEntity(string entityName)
{
	map<string, Entity*>::const_iterator it = entities_.find(entityName);
	if (it == entities_.end()) {
		return NULL;
	} else {
		return it->second;
	}
}

bool KnowledgeModel::hasEntity(string entityName)
{
	return (this->getEntity(entityName) != NULL);
}

bool KnowledgeModel::hasEntity(Entity* e)
{
	return hasEntity(e->getName());
}

string KnowledgeModel::getName()
{
	return name_;
}

void KnowledgeModel::printAllEntities ()
{
	cout << "----------------------------------------" << endl;
	cout << "PRINT ALL ENTITIES IN: " << getName() << ", size: " << entities_.size() << endl;

	for (map<string, Entity*>::const_iterator it = entities_.begin() ; it != entities_.end(); ++it)
	{
		it->second->printContent();
	}
	cout << "----------------------------------------" << endl;
}

void KnowledgeModel::printAllRelations()
{
	cout << "----------------------------------------" << endl;
	cout << "PRINT ALL RELATIONS IN: " << getName() << ", size: " << relations_.size() << endl;

	unsigned int i;
	for (i = 0; i < relations_.size(); i++)
	{
		relations_[i]->printContent();
	}
	cout << "----------------------------------------" << endl;
}
};
