#include "Relation.h"

#include "KnowledgeModel.h"
#include "Entity.h"
#include "RelationType.h"

namespace TCBR_Representation
{
using namespace std;

Relation::Relation(Entity* source, Entity* target, Entity* type, const float strength):
source_(source), target_(target), type_(type), strength_(strength)
{
	knowledgeModel_ = source->getKnowledgeModel();

	knowledgeModel_->addRelation(this);
}

Relation::Relation(Entity* source, Entity* target, string type, const float strength):
source_(source), target_(target), strength_(strength)
{
	knowledgeModel_ = source->getKnowledgeModel();

	if (!knowledgeModel_->hasEntity(type))
	{
		new RelationType (type, knowledgeModel_);
	}
	type_ = knowledgeModel_->getEntity(type);

	knowledgeModel_->addRelation(this);
}

Relation::~Relation()
{
}

Entity* Relation::getSource() const
{
	return source_;
}

Entity* Relation::getTarget() const
{
	return target_;
}

float Relation::getStrength() const
{
	return strength_;
}

Entity* Relation::getType() const
{
	return type_;
}

KnowledgeModel* Relation::getKnowledgeModel() const
{
	return knowledgeModel_;
}

void Relation::printContent()
{
	//cout << " - Relation of type " << getType()->getName() << " with strength "<< getStrength() << endl;
	//cout << "     |--> " << getSource()->getName() << " -> " << getTarget()->getName()  << endl;

	cout << getSource()->getName() << " --(" << getType()->getName() << ")--> " << getTarget()->getName() << endl;
}
};

