#include "Case.h"
#include "CaseBase.h"
#include "SimilarityMeasure.h"

#include <memory>
#include <cassert>
#include <iostream>

using namespace std;

namespace TCBR_Representation
{

double QueryBiasedSymbolicComparator::getSymbolicDifference(Case* from, Case* to, Entity* type)
{
	string name = type->getName();
	if (from->hasFeature(name) && to->hasFeature(name)) {
		Value* a = from->getFeature(name)->getValue();
		Value* b = to->getFeature(name)->getValue();

		// Symbolic value comparisons
		if (a->getTextValue() == b->getTextValue()) {
			return 0.0;
		} else {
			return 1.0;
		}
	} else {
		// Missing value counted the same as different symbolic value
		return 1.0;
	}
}

double QueryBiasedSymbolicComparator::computeCaseSimilarity(Case* from, Case* to)
{
	double distance = 0;

	const vector<Feature*>& fromFeatures(from->getFeatures());
	const vector<Feature*>& toFeatures(to->getFeatures());

	assert(from->getKnowledgeModel() == to->getKnowledgeModel());
	CaseBase* cb = dynamic_cast<CaseBase*>(from->getKnowledgeModel());
	assert(cb);

	// Query-biased comparison: only looking at the features in the from vector
	for (vector<Feature*>::const_iterator it = fromFeatures.begin(); it != fromFeatures.end(); ++it) {
		double weight = cb->getWeight((*it)->getType()->getName());
		Entity* type = (*it)->getType();
		double difference = this->getSymbolicDifference(from, to, type);
		distance += difference * weight;
	}
	double similarity = 1 / (distance + 1);
	return similarity;
}

}
