#include "Value.h"

using namespace std;

/**
 *
 * This value interface should interpret the underlying semantic net
 * and not store any additional information about type
 * All information should come from the semantic net!!
 *
 **/
namespace TCBR_Representation
{

int Value::idCounter_ = 0;

Value::Value(string value, TCBR_Representation::KnowledgeModel* km):
Entity(generateValueID(idCounter_), km)
{
	Relation* rel;
	Entity* ent;
	Entity* thisValue = km->getEntity(getName());

	if(!km->hasEntity("VALUE"))
	{
		new Entity("VALUE", km);
	}

	if(!km->hasEntity("TEXT"))
	{
		new Entity("TEXT", km);
	}

	if (!km->hasEntity(value))
	{
		new Entity(value, km);
	}

	//Make relations to entity VALUE
	ent = km->getEntity("VALUE");

	rel = new Relation(thisValue, ent, "INSTANCE_OF", 1.0);
	thisValue->addRelation(rel);

	rel = new Relation(ent, thisValue, "HAS_INSTANCE", 1.0);
	ent->addRelation(rel);

	//Make relations to entity TEXT
	ent = km->getEntity("TEXT");

	rel = new Relation(thisValue, ent, "HAS_TYPE", 1.0);
	thisValue->addRelation(rel);

	rel = new Relation(ent, thisValue, "TYPE_OF", 1.0);
	ent->addRelation(rel);

	//Make relation to the value
	ent = km->getEntity(value);

	rel = new Relation(thisValue, ent, "HAS_VALUE", 1.0);
	thisValue->addRelation(rel);

	rel = new Relation(ent, thisValue, "VALUE_OF", 1.0);
	ent->addRelation(rel);

	type_ = TYPE_TEXT;
	idCounter_++; //Must be converted to an integer
}

Value::~Value()
{
}

string Value::getTextValue()
{
	string value = "";
	Entity* thisValue = getKnowledgeModel()->getEntity(getName());
	vector<Relation*> rels = thisValue->getLocalRelations("HAS_VALUE");

	if (rels.size() > 0) {
		value = rels.front()->getTarget()->getName();
	}
	return value;
}

int Value::getType()
{
	return type_;
}

//This only prints text values
void Value::printContent()
{
	//Entity* thisValue = getKnowledgeModel()->getEntity(getName());
	cout << "Printing value: " << getName() << endl;
	cout << "The value has type: " << getType() << endl;
	cout << "The actual value is: " << getTextValue() << endl;
}
}
